/**
 * @author Oscar Bahamonde - Luis Vera
 */

import java.util.Vector;
import javax.swing.*; 
import org.jfree.data.xy.*;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;

public class GraphicPanel extends JPanel{

    private JFreeChart chart;
    double maxtime = 30;
    private XYSeriesCollection datasets;
    NumberAxis Xaxis;
    NumberAxis Yaxis;
    private int nsources = 6;
    private XYSeries [] series;
/**
 * Genera grfico
 */
    public GraphicPanel(){
    	series = new XYSeries[nsources+1];
        datasets = new XYSeriesCollection();
        Xaxis = new NumberAxis();
        Yaxis = new NumberAxis();
        Xaxis = new NumberAxis("RTT");  // Label eje X
        Xaxis.setLowerBound(0);
        Xaxis.setUpperBound(maxtime);
        Yaxis = new NumberAxis("congWin"); //Label eje Y
    
        this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
        Xaxis.setAutoRangeIncludesZero(false);
    	Yaxis.setAutoRangeIncludesZero(false);
    	Yaxis.setLowerBound(0);
    	
    	for (int i = 0; i <= nsources; i++) {
    		if (i == 0)
    			series[i] = new XYSeries("Total CongWIn",false);
    		else
    			series[i] = new XYSeries("Source "+i,false);
        }
    	
    	XYPlot plot = new XYPlot(datasets, Xaxis, Yaxis, new StandardXYItemRenderer());
        chart = new JFreeChart("Ventana de Congestion", JFreeChart.DEFAULT_TITLE_FONT, plot, true);
        ChartPanel chartPanel = new ChartPanel(chart);
        add(chartPanel);
    }
/**
 * Agrega datos al grafico
 * @param s Vector de Source (fuentes)
 * @param t Tiempo
 * @param tcw Tamao ventana de congestion
 */
    public void addSeries(Vector<Source> s, double t, double tcw){
    	double sumCongWin = 0;
    	datasets.removeAllSeries();
    	for (int i = 0; i <= s.size(); i++) {
    		if (series[i].getItemCount() > 0 && 
    	    ((series[i].getX(series[i].getItemCount()-1).doubleValue()
                        - series[i].getX(0).doubleValue()) > maxtime)){
                series[i].remove(0);
                Xaxis.setLowerBound(t-maxtime);
                Xaxis.setUpperBound(t);
    		}
        }
    	for (int i = 1; i <= s.size(); i++) {
        	sumCongWin += s.elementAt(i-1).getCongWin();  
        }
        series[0].add(t, sumCongWin);
        datasets.addSeries(series[0]);
        Yaxis.setUpperBound(tcw);
        for (int i = 1; i <= s.size(); i++) {
        	series[i].add(t, s.elementAt(i-1).getCongWin());
        	sumCongWin += s.elementAt(i-1).getCongWin();
        	datasets.addSeries(series[i]);
        }
    }
/**
 * Resetea el grafico
 * @param t Tiempo
 */
    public void reset(double t){
        for (int i = 0; i < nsources; i++) {
            datasets.removeSeries(i);
            datasets.addSeries(new XYSeries("",false));
        }
            Xaxis.setLowerBound(t);
            Xaxis.setUpperBound(t+maxtime);
    }
/**
 * Se setea el valor mximo del eje x
 * @param maxt Tiempo maximo
 */
    public void setMaxTime(double maxt){
        Xaxis.setUpperBound(Xaxis.getLowerBound() + maxt);
        maxtime = maxt;
    }
}
