/**
 * @author  Oscar Bahamonde & Luis Vera
 */
public class Source {
    private double threshold;	//Umbral del nodo.
    private double congWin;		//Ventana del nodo.
    private double nextcongWin;	//Ventana del nodo en el siguiente paso.
    private double bandWidth;	
    private int id;
    private boolean recentlyAdded;	//Nodo recien añadido.
    
    protected Source(int id){
		this.id = id;
	}
    
    /**
     * Constructor. Crear un nuevo nodo.
     * @param id	Identificador del nodo.
     * @param n		Network.
     */
    public Source (int id, myNetwork n) {
    	this.id = id;
        recentlyAdded = true;
        threshold = 0;
        congWin = 1;
        bandWidth = congWin/n.getRTT();
        n.addSource(this);
    }
    
    /**
     * Incrementar la ventana del nodo.
     */
    public void increment () {
        if (recentlyAdded || (2*congWin < threshold))	//Nodo recien añadido o siguiente tamaño de
    	    slowStartIncrement();						//ventana menor al umbral. Partida lenta.
        else if (congWin >= threshold)					//Ventana mayor al umbral.
    	    linearIncrement();							//Incremento lineal.
        else
    	    congWin = threshold;						//Ventana igual al umbral.
    }
    
    /**
     * Simula un incremento de la ventana.
     * @return Siuiente ventana del nodo.
     */
    public double nextIncrement () {
    	if ((threshold == 0) || (2*congWin < threshold))
    	    return 2*congWin;
        else if (congWin >= threshold)
    	    return congWin + 1;
        else
    	    return threshold;
    }
    
    /**
     * Incremento lineal.
     */
    public void linearIncrement () {
        congWin++;
    }
    
    /**
     * Partida lenta.
     */
    public void slowStartIncrement () {
        congWin = 2*congWin;
    }
    
    /**
	 * @return Ventana del nodo.
	 */
    public double getCongWin () {
        return congWin;
    }
    
    /**
     * Nodo recibe 3 Acks repetidos.
     */
    public void threeAck () {
        threshold = congWin/2;	//Umbral igual a la mitad de la ventana del nodo.
        congWin = threshold;	//Ventana igual al umbral. Sigue con Incremento lineal.
    }
    
    /**
     * Nodo recibe Timeout.
     */
    public void timeOut () {
        threshold = congWin/2;	//Umbral igual a la mitad de la ventana del nodo.
        congWin = 1;			//Ventana igual a 1.
    }
    
    /**
     * Obtener identificador del nodo.
     * @return	Identificador.
     */
    public int getID () {
    	return id;
    }
    
    /**
     * Nodo recien añadido a la red.
	 * @param r Booleano que indica que el nodo fue recien añadido.
	 */
    public void setRecentlyAdded(boolean r) {
    	recentlyAdded = r;
    }
    
    /**
     * Pregunta si el nodo fue recien añadido a la red.
	 * @return	Si el nodo fue recien añadido.
	 */
    public boolean getRecentlyAdded() {
    	return recentlyAdded;
    }
}
